using System;

namespace MQ_Postcard
{
	/// <summary>
	/// Summary description for MQUtils.
	/// </summary>
	public class MQUtils {

		public static System.DateTime GetDateTime( byte[] buf, int offset) {
			int i= offset;
			int year= buf[i++]+buf[i++]*256;
			int month= buf[i++]+buf[i++]*256;
			int dayOfWeek= buf[i++]+buf[i++]*256;
			int day=buf[i++]+buf[i++]*256;
			int hour= buf[i++]+buf[i++]*256;
			int minute= buf[i++]+buf[i++]*256;
			int second= buf[i++]+buf[i++]*256;
			int msec= buf[i++]+buf[i++]*256;

			System.DateTime d= System.DateTime.Now;
			try {
			  d= new System.DateTime(year, month, day, hour, minute, second, msec);
			}
			catch (System.ArgumentOutOfRangeException) {
			  
			}
			return d; 
		}
		public static void FillDateTime(ref byte[] buf, int offset) {
						System.DateTime d= System.DateTime.Now;
			int i= offset;
			buf[i++]= (byte) (d.Year % 256);
			buf[i++]= (byte) (d.Year / 256);
			buf[i++]= (byte) (d.Month % 256);
			buf[i++]= (byte) (d.Month / 256);
			buf[i++]= (byte) d.DayOfWeek;
			buf[i++]= 0; 
			buf[i++]= (byte) (d.Day % 256);
			buf[i++]= (byte) (d.Day / 256);
			buf[i++]= (byte) (d.Hour % 256);
			buf[i++]= (byte) (d.Hour/ 256);
			buf[i++]= (byte) (d.Minute % 256);
			buf[i++]= (byte) (d.Minute / 256);
			buf[i++]= (byte) (d.Second % 256);
			buf[i++]= (byte) (d.Second / 256);
			buf[i++]= (byte) (d.Millisecond % 256);
			buf[i++]= (byte) (d.Millisecond / 256);
		}
		/// <summary>
		/// get a string from the byte array
		/// </summary>
		/// <param name="buf">incoming buffer</param>
		/// <param name="start">starting index in the buffer</param>
		/// <param name="maxlength">max length of the returned string</param>
		/// <returns></returns>
		public static string StringFromBuffer(byte[] buf, int start, int maxlength) {
			//			// bogus!  this thing returns a long series of null chars
			//			System.Text.UnicodeEncoding codec= new System.Text.UnicodeEncoding();
			//			return codec.GetString(buf, start, maxlength*2);

			// get the string without the nulls.  
			// (but I don't know if multibyte chars work properly)
			int i;
			char[] c= new char[maxlength];
			for (i=0; (i < maxlength) && (buf[start+i*2]!=0); i++) {
				c[i]= System.BitConverter.ToChar(buf, start+i*2);
			}
			string s=new System.String(c,0,i);
			return s;
		}

		/// <summary>
		/// Fill buffer with byte data from given string.  Start at specified start offset,
		/// and do not exceed maxlength.  
		/// </summary>
		/// <param name="buf">the buffer to fill</param>
		/// <param name="data">string data (double byte chars) to be converted to bytes</param>
		/// <param name="start">start offset</param>
		/// <param name="maxlength">max length of data to fill</param>
		/// 
		private static System.Collections.Hashtable ByteArrayForString= new System.Collections.Hashtable ();
		public static void FillByteArray(ref byte[] buf, string data, int start, int maxlength) {
			byte[] cache;
			int i;
			if (!ByteArrayForString.ContainsKey(data)) {
				// produce the byte array for the given data

				byte [] b;
				cache= new byte[data.Length*2];
				char[] c= data.ToCharArray();
				for (i=0; (i< data.Length) && (i < maxlength/2); i++) {
					b= System.BitConverter.GetBytes(c[i]);
					//buf[start + i*2]= b[0];
					//buf[start + i*2+1]=b[1];
					cache[i*2]= b[0];
					cache[i*2+1]=b[1];
				}
				ByteArrayForString[data]= cache;
			}
			cache= (byte[]) ByteArrayForString[data];

			for (i=0; (i< cache.Length) && (i < maxlength); i++)
				buf[start + i]= cache[i];

			return;
		}
		private static byte[] _MsgId= null;
		public static byte[] MqPostcardMessageId() {
			if (_MsgId!= null) return _MsgId;
			char[] c= "IBMMQSeriesPostCardData ".ToCharArray();
			byte[] result= new byte[24];
			for (int i=0; i < 24; i++) {
				result[i]= System.BitConverter.GetBytes(c[i])[0];
			}
			_MsgId= result;
			return result;
		}



		const int CO_ID_LENGTH= 24; 
		public static byte[] StringToCorrelationId(String s) {
			char[] c=s.ToCharArray();
			byte[] result= new byte[CO_ID_LENGTH];
			int i;
			for (i=0; (i < c.Length) && (i < CO_ID_LENGTH-1); i++) 
				result[i]= System.BitConverter.GetBytes(c[i])[0];

			result[i++]= 0;
			while (i < CO_ID_LENGTH) result[i++]=(byte)0x20;
			return result ;
		}

		public static string PlatformToString(int platformId) {
			string result=platformId.ToString();
			switch (platformId) {
				case 11: result= "Windows";
					break;
			}
			return result;
		}
		public static string MQExceptionToString(IBM.WMQ.MQException mqex) {
			if (mqex==null) return null;
			string m1= mqex.ToString();
			string m2= "???"; 
			if (mqex.Reason == 2058)
				m2= "Error with Queue Manager Name";
			else if (mqex.Reason == 2059)
				return "Queue Manager is Not Available";
			else {
				IBM.WMQAX.MQSession s= new IBM.WMQAX.MQSession();
				m2= s.ReasonCodeName(mqex.Reason);
			}

			return m2 + " (" + m1 + ")";
		}
		public MQUtils() {
			//
			// TODO: Add constructor logic here
			//
		}
	}
}
